/***
   attriubute.c
***/

#define GNU_SOURCE
#include <unistd.h>

#include <credentials/keys/private_key.h>
#include "creddy_internal.h"

extern abac_aspect_t *abac_yy_get_rule_head_aspect();
extern abac_aspect_t *abac_yy_get_rule_tail_aspect();
extern char *abac_aspect_typed_string_with_condition(abac_aspect_t *ptr);
extern abac_context_t *abac_context_new();
extern void abac_context_load_principals(abac_context_t *ctx, char *path);
extern void abac_context_free(abac_context_t *);

static int debug=0;

void attribute_main(options_t *opts) {
    int i, ret, role_len = 1;
    char *head_string=NULL;
    char *subject_string=NULL;

    if ( opts->issuer == NULL || opts->key == NULL ||
                     opts->role == NULL || opts->out == NULL)
        usage(opts);

    abac_context_t *ctx = abac_context_new();
    char *loc=opts->keystore;
    if(loc == 0) {
       char *tmp=NULL;
       loc=getcwd(tmp, 0);
       if(loc==NULL)
         errx(1, "Can't find the keystore's path");
    }

    /* issuer */
    abac_id_t *issuer=abac_id_from_file(opts->issuer);
    if(issuer==NULL)
        errx(1, "Can't load issuer's id from %s", opts->issuer);
    if(opts->passphrase) {
       ret = abac_id_load_enc_privkey_file(issuer, opts->key, opts->pfile);
    } else ret = abac_id_load_privkey_file(issuer, opts->key);

    if (!ret)
        errx(1, "Can't load private key from %s", opts->key);

    abac_context_load_id_id(ctx, issuer);

    /* check on id & pkey */
    if(opts->cred_type == e_cred_role) {
        asprintf(&head_string,"[keyid:%s].role:%s",abac_id_keyid(issuer),opts->role);
        } else {
            asprintf(&head_string,"[keyid:%s].oset:%s",abac_id_keyid(issuer),opts->oset);
    }

    /* load all other principals */
    abac_context_load_principals(ctx, loc);

    /* create the attribute string */
    /* yyparse it, retrieve head/tail aspect */
    /* create abac_attribute_t with it */
    /* bake it.. */
    /* store it into file */

/* XXX To Think More,
   do we want to load the id into context from here ?? if want to, need to
   call, 
   abac_id_credential_t *abac_verifier_add_id_credential(abac_id_t *a_id)
   what if there is already one inside ?? what about its key ??
*/
    for (i = 0; i < opts->num_subjects; ++i) {
        char *string=NULL;
        subject_t *cur = &opts->subjects[i];

        // if we have a cert we need to get its ID
        if (cur->subj_type == e_subj_cert) {
            abac_id_t *subject = abac_id_from_file(cur->cert);
            if (subject == NULL)
                errx(1, "Can't load subject cert from %s", cur->cert);
            cur->id = xstrdup(abac_id_keyid(subject));
            abac_id_free(subject);
        }

        // just a principal, add it
        if (cur->cred_type == e_cred_none) {
            if(cur->subj_type != e_subj_obj) { /*[keyid:id] */
                asprintf(&string,"[keyid:%s]", cur->id);
                } else { /* [type:obj] */
                    string=abac_xstrdup(cur->obj);
            }
        }

/* example, "manageOf([principal:?E[keyid:alpha].role:ownerOf([urn:?F]))" */
        // either role or oset with/without linking role
        else {
            if(cur->cred_type == e_cred_role) {
                if(cur->link!=NULL)
                    asprintf(&string,"[keyid:%s].role:%s.role:%s",
                             cur->id, cur->link, cur->role);
                    else
                        asprintf(&string,"[keyid:%s].role:%s",
                                         cur->id, cur->role);
                } else {
                    if(cur->link!=NULL)
                        asprintf(&string,"[keyid:%s].role:%s.oset:%s",
                                 cur->id, cur->link, cur->oset);
                        else
                            asprintf(&string,"[keyid:%s].oset:%s",
                                             cur->id, cur->oset);
            }

        }
        /* collect up */
        if(subject_string != NULL) {
            char *tmp=subject_string;
            subject_string=NULL;
            asprintf(&subject_string,"%s & %s",tmp,string);
        } else subject_string=string;
    }

    char* attr_string=NULL;
    asprintf(&attr_string,"%s<-%s", head_string, subject_string);
    free(head_string);
    free(subject_string);

    if(debug) printf("attribute_main, attr_string (%s)\n",attr_string);

    /* call into yacc parser */
    abac_reset_yyfptr(attr_string);
    abac_yy_init();
    ret=yyparse();
    if (ret) {
        free(attr_string);
        errx(1, "attribut_main, unable to parse the attr_string");
    }

    abac_aspect_t *head_aspect = abac_yy_get_rule_head_aspect();
    abac_aspect_t *tail_aspect = abac_yy_get_rule_tail_aspect();

    if(debug) {
        char *hstring=abac_aspect_typed_string_with_condition(head_aspect);
        char *tstring=abac_aspect_typed_string_with_condition(tail_aspect);
        printf("hstring(%s)\n",hstring);
        printf("tstring(%s)\n",tstring);
    }

    abac_attribute_t *attr=NULL;
    ret=abac_attribute_create(&attr,head_aspect, tail_aspect, opts->validity);

    if(ret!=ABAC_ATTRIBUTE_SUCCESS)
        errx(1, "attribut_main, unable to make an attribute");

    ret = abac_attribute_bake(attr);
    if (!ret)
        errx(1, "Couldn't bake attribute cert");

    FILE *out = fopen(opts->out, "w");
    if (out == NULL)
        errx(1, "Couldn't open attr cert file %s for writing", opts->out);

    abac_attribute_write_cert(attr, out);

    fclose(out);

    abac_attribute_free(attr);
    abac_context_free(ctx);

}

