#ifndef __CREDDY_H__
#define __CREDDY_H__

#include <library.h>
#include <credentials/certificates/certificate.h>
#include <credentials/certificates/x509.h>
#include <credentials/keys/private_key.h>
#include <credentials/certificates/ac.h>

#include <err.h>
#include <abac.h>

typedef enum { e_cred_none=0, e_cred_role=1, e_cred_oset=2 } ENUM_CRED_TYPE;
typedef enum { e_subj_cert=0, e_subj_id=1, e_subj_obj=2, e_subj_link=3 } ENUM_SUBJ_TYPE;

typedef struct _subject_t {
    ENUM_SUBJ_TYPE subj_type;
    char *obj;
    char *cert;
    char *id;
    char *link;
    ENUM_CRED_TYPE cred_type; 
    char *role; 
    char *oset;
} subject_t;

typedef struct _options_t {
    int help;
    int mode;

    char *cert;
    int passphrase;
    char *pfile;

    // generate options
    char *cn;
    int validity;

    // attribute options
    char *issuer;
    char *key;
    ENUM_CRED_TYPE cred_type; 
    union {
        char *role;
        char *oset;
    };
    subject_t *subjects;
    int num_subjects;
    char *out;
    char *keystore;

    // verify options
    char *attrcert;

    // display options
    char *show;
} options_t;

#define MODE_GENERATE   1
#define MODE_VERIFY     2
#define MODE_KEYID      3
#define MODE_ATTRIBUTE  4
#define MODE_ROLES      5
#define MODE_VERSION    6
#define MODE_DISPLAY    7
#define MODE_OSETS      8
#define MODE_KEYCHECK   9

void usage(options_t *opts);
void *xrealloc(void *ptr, size_t size);
char *xstrdup(char *string);

// sub programs
void generate_main(options_t *opts);
void keyid_main(options_t *opts);
void attribute_main(options_t *opts);
void roles_main(options_t *opts);
void verify_main(options_t *opts);
void display_main(options_t *opts);

// from libabac
void libabac_init();

#endif /* __CREDDY_H__ */
