/*** 
   keycheck.c
***/

#include "creddy.h"
#include <credentials/sets/mem_cred.h>

extern abac_key_t *abac_key_file_new(char *filename, chunk_t potato);
extern private_key_t *abac_key_private_key(abac_key_t *ptr);
extern chunk_t extract_potato(char *filename, char *pfile);

/**
 * Load private key cert with potato.
 */
static int _load_enc_privkey_file(char *filename, chunk_t pp)
{

    abac_key_t *key=abac_key_file_new(filename, pp);
    private_key_t *pkey = abac_key_private_key(key);
    abac_key_free(key);

    if (pkey == NULL) return 0;
    DESTROY_IF(pkey);
    return 1;
}

static int _load_privkey_file(char *filename)
{
    abac_key_t *key=abac_key_file_new(filename, chunk_empty);
    private_key_t *pkey = abac_key_private_key(key);
    abac_key_free(key);
    if (pkey == NULL) return 0;
    DESTROY_IF(pkey);
    return 1;
}

void keycheck_main(options_t *opts) {
    int rc=0;
    chunk_t pp=chunk_empty;

    if (opts->key == NULL)
        usage(opts);

    if(opts->passphrase) {
        pp=extract_potato(opts->key, opts->pfile);
        if(pp.ptr == NULL) errx(1, "Couldn't load get potato from file");
        rc=_load_enc_privkey_file(opts->key, pp);
    } else rc=_load_privkey_file(opts->key);

    if (rc == 0)
        errx(1, "Couldn't load key from %s", opts->key);

    printf("creddy keycheck : %s is okay\n", opts->key);
}

