#include <err.h>
#include <stdio.h>
#include <string.h>
#include <Yap/YapInterface.h>

/*
   ?- anc(Y,mark).
Y = mary ? ;
Y = paul ? ;
Y = bob ? ;
Y = jane ? ;
no
*/


static YAP_Term _buildTermWith2Str(YAP_Functor func, char *str, char *str2)
{
    YAP_Term arg[2];
    arg[0] = YAP_MkAtomTerm(YAP_LookupAtom(str));
    arg[1] = YAP_MkAtomTerm(YAP_LookupAtom(str2));
    return YAP_MkApplTerm(func, 2, arg);
}

static YAP_Term _buildTermWith1Term(YAP_Functor func, YAP_Term fact)
{
    YAP_Term arg[1];
    arg[0] = fact;
    return YAP_MkApplTerm(func, 1, arg);
}

static YAP_Term _buildTermWith2Var(YAP_Functor func)
{
    YAP_Term arg[2];
    arg[0] = YAP_MkVarTerm();
    arg[1] = YAP_MkVarTerm();
    return YAP_MkNewApplTerm(func, 2);
}

/* insert parent facts one at a time
parent(bob, mary).
parent(jane, mary).
parent(mary, mark).
parent(paul, mark).
parent(mark, john).
*/
static void  _build_parent_clauses()
{
// insert each statement one at a time
    YAP_Atom parent = YAP_LookupAtom("parent");
    YAP_Functor func = YAP_MkFunctor(parent, 2);
    YAP_Term fact = _buildTermWith2Str(func, "bob","mary");

    YAP_Atom assert = YAP_LookupAtom("assert");
    YAP_Functor afunc = YAP_MkFunctor(assert, 1);
    YAP_Term goal = _buildTermWith1Term(afunc, fact);
    int res = YAP_RunGoal(goal);
    fact = _buildTermWith2Str(func, "jane","mary");
    goal = _buildTermWith1Term(afunc, fact);
    res = YAP_RunGoal(goal);
    fact = _buildTermWith2Str(func, "mary","mark");
    goal = _buildTermWith1Term(afunc, fact);
    res = YAP_RunGoal(goal);
    fact = _buildTermWith2Str(func, "paul","mark");
    goal = _buildTermWith1Term(afunc, fact);
    res = YAP_RunGoal(goal);
    fact = _buildTermWith2Str(func, "mark","john");
    goal = _buildTermWith1Term(afunc, fact);
    res = YAP_RunGoal(goal);
    printf("done with _build_parent_clauses..\n");
}

/* insert rules one at a time
anc(X, Y):- parent(X, Y).
anc(X, Z):- parent(X, Y), anc(Y, Z).
*/
static void  _build_anc_rules()
{
    YAP_Atom parent = YAP_LookupAtom("parent");
    YAP_Functor pfunc=YAP_MkFunctor(parent, 2);
/* XXX not sure how to sync up the left and right side */
    YAP_Term left = _buildTermWith2Var(pfunc);
    YAP_Atom anc = YAP_LookupAtom("anc");
    YAP_Functor afunc=YAP_MkFunctor(anc, 2);
    YAP_Term right = _buildTermWith2Var(afunc);
    YAP_Term goal;
/* XXX not done 
    int res = YAP_RunGoal(goal);
*/
    printf("calling adding anc rules.. \n");
}

static void _showDB()
{
    char lstr[]="listing";
    YAP_Term *eterm;

    YAP_Term goal=YAP_ReadBuffer(lstr, eterm);
    printf("calling listing.. \n");
    int res =YAP_RunGoal( goal );
    if (res) {
        printf("listing ok.. \n");
    } else {
        printf("listing's res is bad.. \n");
        YAP_Exit(1);
    }
}

/* wrapper used to call into yap */
static int Main_Wrapper() {
    YAP_Term arg[10];
    char str[100];
    char sol[100];
    int i = 0;
    int res;
    int success;
    YAP_Term goal;


    if (YAP_FastInit(NULL) == YAP_BOOT_ERROR)
        YAP_Exit(1);

    if (YAP_RunGoal(YAP_MkAtomTerm(YAP_LookupAtom("source")))) {
        printf("calling source..\n");
        } else {
        printf("calling source failed..\n");
        YAP_Exit(1);
    }

    _build_parent_clauses();
    _build_anc_rules();
/*
    _showDB();
*/

/*
    char cstr[]="consult(main2)";
    YAP_Term *eterm;
    goal=YAP_ReadBuffer(cstr, eterm);
    printf("calling consult.. \n");
    res =YAP_RunGoal( goal );
    if (res) {
        printf("consult ok.. \n");
    } else {
        printf("consult's res is bad.. \n");
        YAP_Exit(1);
    }
*/

/*
    char xstr[]="funnyone(X,Y) :- A(X,Y), B(Y,this)";
    goal=YAP_ReadBuffer(xstr, eterm);
    printf("calling adding a new clause.. \n");
    res =YAP_RunGoal( goal );
    if (res) {
        printf("adding clause ok.. \n");
    } else {
        printf("adding clause's res is bad.. \n");
        YAP_Exit(1);
    }

*/

/*
    YAP_Atom listing = YAP_LookupAtom("listing");
    YAP_Functor lfunc=YAP_MkFunctor(listing, 0);
    goal = YAP_MkApplTerm(lfunc, 0, 0);
    printf("calling listing.. \n");
    res =YAP_RunGoal( goal );
    if (res) {
        printf("listing ok.. \n");
    } else {
        printf("listing's res is bad.. \n");
        YAP_Exit(1);
    }
*/

//  to query : anc(Y, 'str')
    YAP_Atom anc = YAP_LookupAtom("anc");
    YAP_Functor afunc=YAP_MkFunctor(anc, 2);
    for (;;)
        {
        printf("\nEnter a name(mark|mary|bob|jane|john) or 'end' to finish : ");
        fflush(stdout);
        scanf("%s", str);

        if (strcmp(str, "end") == 0)
            break;

        arg[0] = YAP_MkVarTerm();
        arg[1] = YAP_MkAtomTerm(YAP_LookupAtom(str));
        goal = YAP_MkApplTerm(afunc, 2, arg);
        long sl= YAP_InitSlot(goal);
        res = YAP_RunGoal(goal);
        if (res) {
            YAP_Term argterm=YAP_ArgOfTerm(1,YAP_GetFromSlot(sl));
            printf("Success\n");
            YAP_WriteBuffer(argterm, sol, 100,YAP_WRITE_HANDLE_VARS);
            printf("    answer : %s\n", sol);
            while (YAP_RestartGoal()) {
                printf("Success\n");
                YAP_WriteBuffer(argterm, sol, 100,YAP_WRITE_HANDLE_VARS);
                printf("    answer : %s\n", sol);
            }
            } else {
                printf("Failed\n");
        }
        YAP_RecoverSlots(1);
        res=YAP_ShutdownGoal(TRUE);
    }

    return 0;
}

int main(int argc, char *argv[]) {
    return Main_Wrapper();
}
