#!/usr/bin/env python

"""
See README in this directory for the semantics of the example.  This file
constructs the credentials described and puts copies into this directory

cmd1:env keystore=`pwd` ./attr.py 
"""

import os
import ABAC

ctxt = ABAC.Context()
print "ABAC version %s" % ctxt.version()

# Keystore is the directory containing the principal credentials.
# Load existing principals and/or policy credentials
if (os.environ.has_key("keystore")) :
    keystore=os.environ["keystore"]
    ctxt.load_directory(keystore)
else:
    print("keystore is not set...")
    exit(1)


# retrieve principals' keyid value from local credential files
out = ctxt.context_principals()
print "...initial principal set..."
for x in out[1]:
    print "%s " % x.string()
print "\n" 

out = ctxt.context_credentials()
print "...initial policy attribute set..."
for c in out[1]:
    print "%s <- %s" % (c.head_string(), c.tail_string())
print "\n"

aliceID=ABAC.ID("Alice_ID.pem")
aliceID.id_load_privkey_file("Alice_private.pem")
alice=aliceID.id_keyid()

partyID=ABAC.ID("Party_ID.pem")
partyID.id_load_privkey_file("Party_private.pem")
party=partyID.id_keyid()

teaID=ABAC.ID("Tea_ID.pem")
teaID.id_load_privkey_file("Tea_private.pem")
tea=teaID.id_keyid()

hatterID=ABAC.ID("Hatter_ID.pem")
hatterID.id_load_privkey_file("Hatter_private.pem")
hatter=hatterID.id_keyid()

marchhareID=ABAC.ID("Marchhare_ID.pem")
marchhareID.id_load_privkey_file("Marchhare_private.pem")
marchhare=marchhareID.id_keyid()

dormouseID=ABAC.ID("Dormouse_ID.pem")
dormouseID.id_load_privkey_file("Dormouse_private.pem")
dormouse=dormouseID.id_keyid()


# can not have attribute string more than 382 characters or so
# that would cause encoded attribute string more than 511 in length
# and could not push/pull out of ietf_attribute_t properly
###############################################
# NOTE: space and '~' in string value
#       time value's optional ending
# [keyid:Party].role: about([keyid:tea],[time:20101010T],[boolean:true],
#                           [integer:4],[float:-200.0],[float:8],[time:20120205T182930],
#                           [string:'a list'],[urn:'file://usr/party/~teaparty'])
#                <- [keyid:Party].role:guestOf([keyid:MarchHare])
head=ABAC.Role(party,"about")
param=ABAC.DataTerm(teaID)
head.role_add_data_term(param)
param=ABAC.DataTerm("time","20101010T")
head.role_add_data_term(param)
param=ABAC.DataTerm("boolean","true")
head.role_add_data_term(param)
param=ABAC.DataTerm("integer","4")
head.role_add_data_term(param)
param=ABAC.DataTerm("float","-200.0")
head.role_add_data_term(param)
param=ABAC.DataTerm("float","8")
head.role_add_data_term(param)
param=ABAC.DataTerm("time","20120205T182930")
head.role_add_data_term(param)
param=ABAC.DataTerm("string","'a list'")
head.role_add_data_term(param)
param=ABAC.DataTerm("urn","'file://usr/party/~teaparty'")
head.role_add_data_term(param)
tail=ABAC.Role(party,"guestOf")
param=ABAC.DataTerm(marchhareID)
tail.role_add_data_term(param)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_party__guestof_marchhare_attr.der")
ctxt.load_attribute_file("party_about_party__guestof_marchhare_attr.der")
print attr.string() 
print attr.typed_string()
print "\n"

###############################################
# NOTE: anonymous parameter
#       Variable value for different parameter types
# [keyid:Party].role:about_other_party([keyid:tea],[?],
#                         [integer:99],[time:?AAA],[integer:?BBB],
#                         [boolean:true],[integer:?])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_other_party")
param=ABAC.DataTerm(teaID)
head.role_add_data_term(param)
param=ABAC.DataTerm("anonymous","_")
head.role_add_data_term(param)
param=ABAC.DataTerm("integer","99")
head.role_add_data_term(param)
param=ABAC.DataTerm("time","AAA")
head.role_add_data_term(param)
param=ABAC.DataTerm("integer","BBB")
head.role_add_data_term(param)
param=ABAC.DataTerm("boolean","true")
head.role_add_data_term(param)
param=ABAC.DataTerm("integer","?")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_other_party__marchhare_attr.der")
ctxt.load_attribute_file("party_about_other_party__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_string([string:'a confused party'])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_string")
param=ABAC.DataTerm("string","'a confused party'")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_string__marchhare_attr.der")
ctxt.load_attribute_file("party_about_string__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# NOTE: how to escape within a string value
# [keyid:Party].role:about_string2([string:'a mangled \\'string\\''])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_string2")
param=ABAC.DataTerm("string","'a mangled \\'string\\''")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_string2__marchhare_attr.der")
ctxt.load_attribute_file("party_about_string2__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_urn([urn:'file://user/local/party/~sillyparty'])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_urn")
param=ABAC.DataTerm("urn","'file://user/local/party/~sillyparty'")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_urn__marchhare_attr.der")
ctxt.load_attribute_file("party_about_urn__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_another_float([float:0.22])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_another_float")
param=ABAC.DataTerm("float","0.22")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_third_float__marchhare_attr.der")
ctxt.load_attribute_file("party_about_third_float__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_another_float([float:8])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_another_float")
param=ABAC.DataTerm("float","8")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_another_float__marchhare_attr.der")
ctxt.load_attribute_file("party_about_another_float__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_float([float:-200.0])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_float")
param=ABAC.DataTerm("float","-200.0")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_float__marchhare_attr.der")
ctxt.load_attribute_file("party_about_float__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_another_integer([integer:-7])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_another_integer")
param=ABAC.DataTerm("integer","-7")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_another_integer__marchhare_attr.der")
ctxt.load_attribute_file("party_about_another_integer__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_integer([integer:4])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_integer")
param=ABAC.DataTerm("integer","4")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_integer__marchhare_attr.der")
ctxt.load_attribute_file("party_about_integer__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_boolean([boolean:true])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_boolean")
param=ABAC.DataTerm("boolean","true")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_boolean__marchhare_attr.der")
ctxt.load_attribute_file("party_about_boolean__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_key([keyid:tea])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_key")
param=ABAC.DataTerm(teaID)
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_key__marchhare_attr.der")
ctxt.load_attribute_file("party_about_key__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_another_time([time:20201101T182930])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_another_time")
param=ABAC.DataTerm("time","20201101T182930")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_another_time__marchhare_attr.der")
ctxt.load_attribute_file("party_about_another_time__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:about_time([time:20201101T])
#                 <- [keyid:MarchHare]
head=ABAC.Role(party,"about_time")
param=ABAC.DataTerm("time","20201101T")
head.role_add_data_term(param)
tail=ABAC.Role(marchhare)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_about_time__marchhare_attr.der")
ctxt.load_attribute_file("party_about_time__marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# NOTE: principal parameter type
# [keyid:Party].role:guestof([principal:?V]) 
#           <- [keyid:Party].role:friendOf([principal:?V])
#
head=ABAC.Role(party,"guestOf")
param=ABAC.DataTerm("principal","V")
head.role_add_data_term(param)
tail=ABAC.Role(party,"friendOf")
param=ABAC.DataTerm("principal","V")
tail.role_add_data_term(param)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_guestof_qV__party_friendof_qV_attr.der")
ctxt.load_attribute_file("party_guestof_qV__party_friendof_qV_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:guests 
#        <- [keyid:Party].role:friendOf([keyid:MarchHare])
#
head=ABAC.Role(party,"guests")
tail=ABAC.Role(party,"friendOf")
param=ABAC.DataTerm(marchhareID)
tail.role_add_data_term(param)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_guests__party_friendof_marchhare_attr.der")
ctxt.load_attribute_file("party_guests__party_friendof_marchhare_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# [keyid:Party].role:friendOf([keyid:MarchHare]) 
#                               <- [keyid:Dormouse]
#
head=ABAC.Role(party,"friendOf")
param=ABAC.DataTerm(marchhareID)
head.role_add_data_term(param)
tail=ABAC.Role(dormouse)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_friendof_marchhare__dormouse_attr.der")
ctxt.load_attribute_file("party_friendof_marchhare__dormouse_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

###############################################
# NOTE: named principal tail
# [keyid:Party].role:friendOf([keyid:Alice]) 
#                               <- [keyid:Hatter]
#
head=ABAC.Role(party,"friendOf")
param=ABAC.DataTerm(aliceID)
head.role_add_data_term(param)
tail=ABAC.Role(hatter)
attr=ABAC.Attribute(head, 1800)
attr.attribute_add_tail(tail)
attr.attribute_bake()
attr.attribute_write_cert("party_friendof_alice__hatter_attr.der")
ctxt.load_attribute_file("party_friendof_alice__hatter_attr.der")
print attr.string()
print attr.typed_string()
print "\n"

