# ===========================================================================
# ===========================================================================
#
# SYNOPSIS
#
#   AC_PROG_GPROLOG([major.minor.micro])
#
# DESCRIPTION
#
#   This macro searches for a gprolog installation on your system. If found you
#   should call gprolog via $(GPROLOG). You can use the optional first argument to
#   check if the version of the available gprolog is greater than or equal to
#   the value of the argument. It should have the format: N[.N[.N]] (N is a
#   number between 0 and 999. Only the first N is mandatory.)
#
#   If the version argument is given (e.g. 1.3.17), AC_PROG_GPROLOG checks that
#   the swig package is this version number or higher.
#
#   In configure.in, use as:
#
#     AC_PROG_GPROLOG(1.3.17)
#

AC_DEFUN([AC_PROG_GPROLOG],[
        AC_PATH_PROG([GPROLOG],[gprolog])
        if test -z "$GPROLOG" ; then
                AC_MSG_WARN([cannot find 'gprolog' program. You should look at http://www.gprolog.org])
                GPROLOG='echo "Error: gprolog is not installed. You should look at http://www.gprolog.org" ; false'
        elif test -n "$1" ; then
                AC_MSG_CHECKING([for GPROLOG version])
                [gprolog_version=`$GPROLOG --version 2>&1 | grep 'GNU Prolog' | sed 's/.*\([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\).*/\1/g'`]
                AC_MSG_RESULT([$gprolog_version])
                if test -n "$gprolog_version" ; then
                        # Calculate the required version number components
                        [required=$1]
                        [required_major=`echo $required | sed 's/[^0-9].*//'`]
                        if test -z "$required_major" ; then
                                [required_major=0]
                        fi
                        [required=`echo $required | sed 's/[0-9]*[^0-9]//'`]
                        [required_minor=`echo $required | sed 's/[^0-9].*//'`]
                        if test -z "$required_minor" ; then
                                [required_minor=0]
                        fi
                        [required=`echo $required | sed 's/[0-9]*[^0-9]//'`]
                        [required_patch=`echo $required | sed 's/[^0-9].*//'`]
                        if test -z "$required_patch" ; then
                                [required_patch=0]
                        fi
                        # Calculate the available version number components
                        [available=$gprolog_version]
                        [available_major=`echo $available | sed 's/[^0-9].*//'`]
                        if test -z "$available_major" ; then
                                [available_major=0]
                        fi
                        [available=`echo $available | sed 's/[0-9]*[^0-9]//'`]
                        [available_minor=`echo $available | sed 's/[^0-9].*//'`]
                        if test -z "$available_minor" ; then
                                [available_minor=0]
                        fi
                        [available=`echo $available | sed 's/[0-9]*[^0-9]//'`]
                        [available_patch=`echo $available | sed 's/[^0-9].*//'`]
                        if test -z "$available_patch" ; then
                                [available_patch=0]
                        fi
                        if test $available_major -ne $required_major \
                                -o $available_minor -ne $required_minor \
                                -o $available_patch -lt $required_patch ; then
                                AC_MSG_WARN([GPROLOG version >= $1 is required.  You have $gprolog_version.  You should look at http://www.gprolog.org])
                                GPROLOG='echo "Error: GPROLOG version >= $1 is required.  You have '"$gprolog_version"'.  You should look at http://www.gprolog.org" ; false'
                        else
                                AC_MSG_NOTICE([GPROLOG executable is '$GPROLOG'])
                        fi
                else
                        AC_MSG_WARN([cannot determine gprolog version])
                        GPROLOG='echo "Error: Cannot determine gprolog version.  You should look at http://www.gprolog.org" ; false'
                fi
        fi
        AC_SUBST([GPROLOG])

        #
        # Check for GPLC 
        #
        AC_PATH_PROG([GPLC],[gplc])
        if test -z "$GPLC" ; then
                AC_MSG_WARN([cannot find 'gplc' program. You should look at http://www.gprolog.org])
                GPROLOG='echo "Error: gplc is not installed. You should look at http://www.gprolog.org" ; false'
        fi
        AC_SUBST([GPLC])
])
