# ===========================================================================
# ===========================================================================
#
# SYNOPSIS
#
#   AC_PROG_YAP([major.minor.micro])
#
# DESCRIPTION
#
#   This macro searches for a YAP prolog installation on your system. If found you
#   should call YAP prolog via $(YAP). You can use the optional first argument to
#   check if the version of the available YAP is greater than or equal to
#   the value of the argument. It should have the format: N[.N[.N]] (N is a
#   number between 0 and 999. Only the first N is mandatory.)
#
#   If the version argument is given (e.g. 1.3.17), AC_PROG_YAP checks that
#   the swig package is this version number or higher.
#
#   In configure.in, use as:
#
#     AC_PROG_YAP(1.3.17)
#

AC_DEFUN([AC_PROG_YAP],[
        AC_PATH_PROG([YAP],[yap])
        if test -z "$YAP" ; then
                AC_MSG_WARN([cannot find yap program. You should look at http://www.dcc.fc.up.pt/~vsc/YAP/])
                YAP='echo "Error: yap is not installed. You should look at http://www.dcc.fc.up.pt/~vsc/Yap/" ; false'
        elif test -n "$1" ; then
                AC_MSG_CHECKING([for YAP version])
                [yap_version=`$YAP --version 2>&1 | grep 'YAP ' | sed 's/.*\([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\).*/\1/g'`]
                AC_MSG_RESULT([$yap_version])
                if test -n "$yap_version" ; then
                        # Calculate the required version number components
                        [required=$1]
                        [required_major=`echo $required | sed 's/[^0-9].*//'`]
                        if test -z "$required_major" ; then
                                [required_major=0]
                        fi
                        [required=`echo $required | sed 's/[0-9]*[^0-9]//'`]
                        [required_minor=`echo $required | sed 's/[^0-9].*//'`]
                        if test -z "$required_minor" ; then
                                [required_minor=0]
                        fi
                        [required=`echo $required | sed 's/[0-9]*[^0-9]//'`]
                        [required_patch=`echo $required | sed 's/[^0-9].*//'`]
                        if test -z "$required_patch" ; then
                                [required_patch=0]
                        fi
                        # Calculate the available version number components
                        [available=$yap_version]
                        [available_major=`echo $available | sed 's/[^0-9].*//'`]
                        if test -z "$available_major" ; then
                                [available_major=0]
                        fi
                        [available=`echo $available | sed 's/[0-9]*[^0-9]//'`]
                        [available_minor=`echo $available | sed 's/[^0-9].*//'`]
                        if test -z "$available_minor" ; then
                                [available_minor=0]
                        fi
                        [available=`echo $available | sed 's/[0-9]*[^0-9]//'`]
                        [available_patch=`echo $available | sed 's/[^0-9].*//'`]
                        if test -z "$available_patch" ; then
                                [available_patch=0]
                        fi
                        if test $available_major -ne $required_major \
                                -o $available_minor -lt $required_minor \
                                -o $available_patch -lt $required_patch ; then
                                AC_MSG_WARN([YAP version >= $1 is required.  You have $yap_version.  You should look at http://www.dcc.fc.up.pt/~vsc/Yap/])
                                YAP='echo "Error: YAP version >= $1 is required.  You have '"$yap_version"'.  You should look at http://www.dcc.fc.up.pt/~vsc/Yap/" ; false'
                        else
                                AC_MSG_NOTICE([YAP executable is '$YAP'])
                                YAP_VERSION=$yap_version
                                AC_SUBST([YAP_VERSION])
                        fi
                else
                        AC_MSG_WARN([cannot determine yap version])
                        YAP='echo "Error: Cannot determine yap version.  You should look at http://www.dcc.fc.up.pt/~vsc/Yap/" ; false'
                fi
        fi
        AC_SUBST([YAP])
])
